%{
  open Ast
  open Printf
  open Lexing
%}

%token <int> INT
%token <string> VAR
%token PLUS EOF LPAREN RPAREN COMMA DOT COLON TRUE FALSE EQUALS END
       IF THEN ELSE FST SND LAMBDA LET IN INTTYP BOOLTYP ARROW

%type <typ> t 
%type <expr> e
%type <Ast.expr> program

%left PLUS 
%nonassoc EQUALS
%left COMMA
%left STAR
%left ARROW

%start program
 
%%

t : INTTYP           { IntType }
  | BOOLTYP          { BoolType }
  | t ARROW t        { ArrowType($1, $3) }
  | t STAR t         { ProdType($1, $3) }
  | LPAREN t RPAREN  { $2 }

e : INT                           {Int($1)}
  | VAR                           {Var($1)}
  | TRUE                          {True}
  | FALSE                         {False}
  | LPAREN IF e e e RPAREN        {If($3, $4, $5)}
  | LPAREN e PLUS e RPAREN        {Plus($2, $4)}
  | LPAREN e COMMA e RPAREN       {Pair($2, $4)}
  | LPAREN FST e RPAREN           {Fst($3)}
  | LPAREN SND e RPAREN           {Snd($3)}
  | LPAREN e e  RPAREN            {Apply($2, $3)}
  | LAMBDA VAR DOT e              {Lambda($2, $4)}
  | LET VAR EQUALS e IN e END     {Let($2, $4, $6)}
  | LAMBDA VAR COLON t DOT e      {TypedLambda($2, $4, $6)}
  | LPAREN e RPAREN               {$2}

program : e                       {$1}
