open Ast

let rec
  pprintType t = pprintType'(0, t)

and pprintExpr e = pprintExpr'(0, e)

and print = print_string

and space(n) = if n = 0 then () else (print " "; space(n-1))

and pprintType' (n, e) = match e with
  | (IntType) -> print "int"

  | (BoolType) -> print "bool"

  | (ArrowType(t1,t2)) ->
  (print "(";pprintType'(n, t1); print " -> "; pprintType'(n, t2); print ")")

  | (ProdType(t1,t2)) ->
  (print "(";pprintType'(n, t1); print " * "; pprintType'(n, t2); print ")")

  | (TypeVar(s)) -> (print "'";print s)

and pprintExpr' (n, e) = match e with
  | (True) -> print "true"

  | (False) -> print "false"

  | (Var x) -> print x

  | (Int(i)) -> print (string_of_int i)

  | (If(b,e1,e2)) ->
  (print "(if "; pprintExpr'(n,b); print " then ";
   pprintExpr'(n,e1); print " else "; pprintExpr'(n,e2); print ")")

  | (Plus(e1, e2))  ->
  (print "("; pprintExpr'(n, e1); print " + "; pprintExpr'(n, e2); print ")")

  | (Pair(e1, e2))  ->
  (print "("; pprintExpr'(n, e1); print ", "; pprintExpr'(n, e2); print ")")

  | (Fst(e))  -> (print "(fst "; pprintExpr'(n, e); print ")")

  | (Snd(e))  -> (print "(snd "; pprintExpr'(n, e); print ")")

  | (Apply(e1, e2))  ->
  (print "("; pprintExpr'(n, e1); print " "; pprintExpr'(n, e2); print ")")

  | (Lambda(x, e))  ->
  (print "(lambda "; print x; print ". "; pprintExpr'(n, e); print ")")

  | (TypedLambda(x, t, e))  ->
  (print "(lambda "; print x; print " : ";
   pprintType'(n, t);  print ". "; pprintExpr'(n, e); print ")")

  | (Let(x, e1, e2))  ->
  (print "let "; print x; print " = "; pprintExpr'(n, e1);
   print " in \n"; space (n+2); pprintExpr'(n+2, e2);
   print "\n"; space n; print " end")

